@@ -81,7 +81,7 @@ pub fn blink_custom(on_time: u64, off_time: u64) {
 pub fn debug_hex(hex: u32, message: &[u8]) {
     if DEBUG_SERIAL_ENABLED {
         serial_write(SerioDevice::Debug, b"0x");
-        serial_write_vec(SerioDevice::Debug, &to_base(hex as u64, 16));
+        serial_write_str(SerioDevice::Debug, &itob(hex as u64, 16));
         serial_write(SerioDevice::Debug, b" ");
         debug_str(message);
     }
@@ -90,16 +90,7 @@ pub fn debug_hex(hex: u32, message: &[u8]) {
 /// Write out a u64 number and a string of data to Uart4
 pub fn debug_u64(val: u64, message: &[u8]) {
     if DEBUG_SERIAL_ENABLED {
-        serial_write_vec(SerioDevice::Debug, &itoa_u64(val));
-        serial_write(SerioDevice::Debug, b" ");
-        debug_str(message);
-    }
-}
-
-/// Write out a u32 number and a string of data to Uart4
-pub fn debug_u32(val: u32, message: &[u8]) {
-    if DEBUG_SERIAL_ENABLED {
-        serial_write_vec(SerioDevice::Debug, &to_base(val as u64, 10));
+        serial_write_str(SerioDevice::Debug, &itoa(val));
         serial_write(SerioDevice::Debug, b" ");
         debug_str(message);
     }
