@@ -3,5 +3,5 @@ pub mod buffer;
 pub mod closure;
 pub mod map;
 pub mod observable;
-pub mod strings;
+pub mod str;
 pub mod vector;
